package com.cj.jiaqingjiayi.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.model.domain.Commodities;
import com.cj.jiaqingjiayi.model.domain.OrderItems;
import com.cj.jiaqingjiayi.model.domain.Orders;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.order.OrderAddRequest;
import com.cj.jiaqingjiayi.model.request.order.OrderItemsAddRequest;
import com.cj.jiaqingjiayi.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 订单相关接口
 **/
@Slf4j
@RestController
@Api(tags = "订单接口")
@RequestMapping("/orders")
public class OrdersController {

    @Resource
    private UserService userService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private BusinessService businessService;

    @Resource
    private OrderItemsService orderItemsService;

    @Resource
    private CommoditiesService commoditiesService;


    /**
     * 创建订单
     *
     * @return 订单id
     */
    @ApiOperation(value = "订单创建接口")
    @PostMapping("/add")
    public BaseResponse<Long> addOrders(@RequestBody OrderAddRequest orderAddRequest, HttpServletRequest request) {
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 提取订单
        Orders orders = new Orders();
        User loginUser = userService.getLoginUser(request);
        BeanUtils.copyProperties(orderAddRequest, orders);
        orders.setUserId(loginUser.getId());
        ordersService.validOrder(orders);
        // 提取订单详情
        List<OrderItemsAddRequest> detailAddRequest = orderAddRequest.getOrderItemsAddRequest();
        List<OrderItems> orderItemsList = detailAddRequest.stream().map(item -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(item, orderItems);
            orderItemsService.validOrderItems(orderItems);
            Long commoditiesId = orderItems.getCommoditiesId();
            // 根据菜品设置价格
            Commodities commodities = commoditiesService.getById(commoditiesId);
            BigDecimal commoditiesPrice = BigDecimal.valueOf(commodities.getCommoditiesPrice());
            orderItems.setPrice(commoditiesPrice);
            double subtotal = commodities.getCommoditiesPrice() * orderItems.getQuantity();
            orderItems.setSubtotal(BigDecimal.valueOf(subtotal));
            return orderItems;
        }).toList();
        // 创建订单
        long orderId = ordersService.addOrder(orders, orderItemsList);
        return ResultUtils.success(orderId, "订单创建成功");
    }




}


